/*******************************************************************************
* Filename : VRBot.pde
* Arduino 2011-VRBot control program 
* for ATMEGA 328 and ATMEGA 168 with Arduino version 0021
* The program implements a simple way of controlling a robot serially by using
* the words: "forward", "backwads", "left" and "right"
* By Carlos Asmat at RobotShop based on the VRbot Sample Arduino code by Veear.
********************************************************************************/

#include <SoftwareSerial.h>
#include "protocol.h"

uint8_t _receivePin;
uint8_t _transmitPin;
long _baudRate;
int _bitPeriod;

void setup(){  
  
  // PC UART Init
  pinMode(0, INPUT);         // sets the digital pin as input
  pinMode(1, OUTPUT);        // sets the digital pin as output

  // VRbot UART Init
  pinMode(11, INPUT);        // sets the digital pin as input
  pinMode(12, OUTPUT);       // sets the digital pin as output
  
  Serial.begin(9600);
  delay(200);
  
  // Set up and detect VRbot
  VRbot_setup();  
  
  if (!VRbot_Detect())                 
     Serial.println("VRbot NA");
  else {
       //Serial.println("VRbot detected");
       // Set VRbot timeout to 2 seconds
       VRbot_SetTimeout(1);
       delay(200);
       
       // Set VRbot language to English       
       //Serial.println("Setting Language to: English");
       VRbot_SetLanguage(0);
  }  
}

void loop(){ 
  
  SI_Recognition(); 
}

/*******************************************************************************
* Speaker Independent recognition function:
*
* Wait for trigger word 'Robot'
* Set wordset 1 
* Wait for command MOVE or command TURN, errors or other commands are ignored
* Set wordset 2
* Wait for command FOWRWARD or command BACWARD for MOVE, errors or other commands are ignored
* or
* Wait for command LEFT or command RIGHT for TURN, errors or other commands are ignored
* 
********************************************************************************/

void SI_Recognition()
{
  int cmd;
  
Serial.println("l");
VRbot_RecognizeSI(2);       // start SI recognition wordset 2
cmd = VRbot_CheckResult();  // check recognition result

switch (cmd){
  case -2: // Error
    Serial.println("E");
    break;

  case -1: // Timoeut
    Serial.println("T");
    break;
    
  case 4:  // FORWARD
    Serial.print("w");
    delay (1000);
    Serial.print("f");       
    break;

  case 5:  // BACWARD
    Serial.print("s");
    delay (1000);
    Serial.print("f");
    break;
    
 case 0: // LEFT
    Serial.print("a");
    delay (250);
    Serial.print("f");
    break;
    
  case 1:// RIGHT
    Serial.print("d");
    delay (250);
    Serial.print("f");
    break;

  default: // Other command
    Serial.println("other");
    break;
    
  delay (100);
  }
  
}

void VRbot_setup()
{
  _baudRate = 9600;
  _bitPeriod = 1000000 / _baudRate;
  _receivePin = 11;
  _transmitPin = 12;
  digitalWrite(_transmitPin, HIGH);
  delayMicroseconds( _bitPeriod); 
}

unsigned char VRbot_read(void)
{
  uint8_t val = 0;
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(100);
  
  // one byte of serial data (LSB first)
  // ...--\    /--\/--\/--\/--\/--\/--\/--\/--\/--...
  //	 \--/\--/\--/\--/\--/\--/\--/\--/\--/
  //	start  0   1   2   3   4   5   6   7 stop

  while (digitalRead(_receivePin));

  // confirm that this is a real start bit, not line noise
  if (digitalRead(_receivePin) == LOW) {
    // frame start indicated by a falling edge and low start bit
    // jump to the middle of the low start bit
    delayMicroseconds(bitDelay / 2 - clockCyclesToMicroseconds(50));
	
    // offset of the bit in the byte: from 0 (LSB) to 7 (MSB)
    for (int offset = 0; offset < 8; offset++) {
	// jump to middle of next bit
	delayMicroseconds(bitDelay);
	
	// read bit
	val |= digitalRead(_receivePin) << offset;
    }
	
    delayMicroseconds(_bitPeriod);
    
    return val;
  }
  
  return -1;
}

void VRbot_write(uint8_t b)
{
  if (_baudRate == 0)
    return;
    
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50); // a digitalWrite is about 50 cycles
  byte mask;

  digitalWrite(_transmitPin, LOW);
  delayMicroseconds(bitDelay);

  for (mask = 0x01; mask; mask <<= 1) {
    if (b & mask){ // choose bit
      digitalWrite(_transmitPin,HIGH); // send 1
    }
    else{
      digitalWrite(_transmitPin,LOW); // send 1
    }
    delayMicroseconds(bitDelay);
  }

  digitalWrite(_transmitPin, HIGH);
  delayMicroseconds(bitDelay);
}

/*******************************************************************************/

unsigned char VRbot_Detect(void) {
  unsigned char i;
  for (i = 0; i < 5; ++i) {
    delay(100);    
    VRbot_write(CMD_BREAK);        
    if ( VRbot_read() == STS_SUCCESS)
      return 255;
  }
  return 0;
}

unsigned char VRbot_SetLanguage(unsigned char lang) {
        
  VRbot_write(CMD_LANGUAGE);
  delay(5);
  VRbot_write(ARG_ZERO + lang);

  if (VRbot_read() == STS_SUCCESS)
    return 255;
  return 0;
}

void VRbot_RecognizeSD(unsigned char group) {
  VRbot_write(CMD_RECOG_SD);
  delay(5);
  VRbot_write(ARG_ZERO + group);
}

void VRbot_RecognizeSI(unsigned char ws) {
  VRbot_write(CMD_RECOG_SI);
  delay(5);
  VRbot_write(ARG_ZERO + ws);
}

void VRbot_SetTimeout(unsigned char s) {
  VRbot_write(CMD_TIMEOUT);
  delay(5);
  VRbot_write(ARG_ZERO + s);
  delay(5);
}

signed char VRbot_CheckResult(void) {
  
  unsigned char rx;
  rx = VRbot_read();
  if (rx == STS_SIMILAR || rx == STS_RESULT) {
    delay(5);
    VRbot_write(ARG_ACK);
    return (VRbot_read() - ARG_ZERO);
  }
  if (rx == STS_TIMEOUT)
    return -1;
    
  return -2;
}
